@Id
{% if column_name %}
    @Column(name = "{{ column_name }}")
{% else %}
    @Column
{% endif %}
{% if strategy == 'sequence' %}
    @SequenceGenerator(name = "{{ id.sequence }}", sequenceName = "{{ id.sequence }}", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "{{ id.sequence }}")
{% elif strategy != 'none' %}
    @GeneratedValue(strategy = GenerationType.{{ id.generation|upper }})
{% endif %}
private {{ class_type }} {{ name }};
