package {{ package }};

{# Fixed imports in the temlate #}
import lombok.*;

{{ imports }}


@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class {{ class_type }} {

{% filter indent(width=4) %}
    {{ fields }}
{%- endfilter -%}
}
